import { action } from '@storybook/addon-actions';
import { Meta } from '@storybook/react';
import React, { FC, useState } from 'react';

import {
  mergeCallbacks,
  unselect,
  Unselect,
  useListSelection,
  useListStyle,
  DefaultList,
  ListSelectionProvider,
  ListStyleProvider,
} from '@clds/compound-components';
// eslint-disable-next-line @nx/enforce-module-boundaries
import {
  Edit,
  Extension,
  FaceDetect,
  Settings,
  SharedCollection,
  Transform,
} from '@clds/icon';
// eslint-disable-next-line @nx/enforce-module-boundaries
import { Item, ItemLayout, ItemProps } from '@clds/item';

import { NestedListItemsWrapper } from './common';

const ListItem: FC<ItemProps & { id: string }> = ({
  id,
  role = 'listitem',
  onClick,
  children,
  ...rest
}) => {
  const listContext = useListStyle();
  const { onClick: useListSelectionOnClick, ...selectionContext } =
    useListSelection(id);

  return (
    <Item
      data-id={id}
      role={role}
      {...listContext}
      {...selectionContext}
      {...rest}
      onClick={mergeCallbacks(onClick, useListSelectionOnClick)}
    >
      {children}
    </Item>
  );
};

const Demo = () => {
  const [selectedId, setSelectedId] = useState<string | Unselect>(unselect);

  return (
    <>
      <p>
        <code>selectedId: {String(selectedId)}</code>
      </p>
      <ListSelectionProvider onClick={setSelectedId} selectedId={selectedId}>
        <ListStyleProvider borderRadius="xl">
          <DefaultList>
            <ListItem id="settings" onClick={action('manage clicked')}>
              <ItemLayout leftSlot={<Settings />}>Manage</ItemLayout>
            </ListItem>
            <ListItem
              id="edit-transformations"
              onClick={action('edit transformations clicked')}
            >
              <ItemLayout leftSlot={<Transform />}>
                Edit Transformations
              </ItemLayout>
            </ListItem>
            <ListItem id="analyze" onClick={action('analyze clicked')}>
              <ItemLayout leftSlot={<FaceDetect />}>Analyze</ItemLayout>
            </ListItem>
            <NestedListItemsWrapper>
              <p>
                {/* eslint-disable-next-line react/no-unescaped-entities */}
                You can see that list items don't need to be immediate children
                of List component
              </p>
              <ListItem id="edit" onClick={action('edit clicked')}>
                <ItemLayout leftSlot={<Edit />}>Edit</ItemLayout>
              </ListItem>
              <ListItem id="extension" onClick={action('extension clicked')}>
                <ItemLayout leftSlot={<Extension />}>Extension</ItemLayout>
              </ListItem>
            </NestedListItemsWrapper>
            <ListItem id="share" onClick={action('share clicked')}>
              <ItemLayout leftSlot={<SharedCollection />}>Share</ItemLayout>
            </ListItem>
          </DefaultList>
        </ListStyleProvider>
      </ListSelectionProvider>
    </>
  );
};

export const ConnectedComponents = () => (
  <>
    <h1>Connected components</h1>
    <p>
      Once you know pure components you may find them not very practical in real
      life.
    </p>
    <p>
      This story demonstrates how we use <em>context</em> to{' '}
      <em>communicate</em> components in order to implement:
    </p>
    <ul>
      <li>
        easy configuration of list variant style (rounded, striped, ...) in one
        place
      </li>
      <li>single-mode selection on item click</li>
    </ul>
    <p>
      Please look at actions panel to check click handlers. Also please look at
      controls panel to control list variant.
    </p>
    <Demo />
    <h4>Easily set variant related props to pure components</h4>
    <p>
      In order to avoid configuring each pure component individually, we provide
      list variant props via <strong>ListStyleProvider</strong> and connect pure
      components to this context so they all can consume the same props{' '}
    </p>
    <h4>Easily manage selection state</h4>
    <p>
      In order to avoid setting <code>selected</code> and <code>onClick</code>{' '}
      properties on each single menu item, we provide selectedId and onClick(id)
      via <strong>ListSelectionProvider</strong> and connect pure item
      components to this context. Connected item components expect{' '}
      <code>id</code>
      property in order to determine if they should be selected or not.
      Connected component also translates received onClick handler into call to
      {/* eslint-disable-next-line react/no-unescaped-entities */}
      context's onClick(id).
    </p>
    <p>
      <strong>ListSelectionProvider</strong> expects <code>selectedId</code> and{' '}
      <code>onClick(id)</code>. This is convenient when you want to store
      selected item state using useState() hook.
    </p>
    <h2>Drawbacks</h2>
    <p>
      At this stage we need to pass item id to list item only to handle
      selection. This is not very convenient when it is not needed elsewhere.
    </p>
  </>
);

const meta: Meta<typeof Demo> = {
  title: 'Compound Components/02 Connected List',
  component: Demo,
};

export default meta;
