import { action } from '@storybook/addon-actions';
import { Meta } from '@storybook/react';
import React, { FC, useState } from 'react';

import {
  KeyboardHandlers,
  useListNavigationHandlers,
  DefaultList,
  ListSelectionProvider,
  useListSelection,
  ListStyleProvider,
  useListStyle,
  useListNavigation,
  mergeCallbacks,
  PubSubProvider,
  usePubSub,
  usePubSubSubscribe,
  unselect,
  Unselect,
} from '@clds/compound-components';
// eslint-disable-next-line @nx/enforce-module-boundaries
import {
  Edit,
  Extension,
  FaceDetect,
  Settings,
  SharedCollection,
  Transform,
} from '@clds/icon';
// eslint-disable-next-line @nx/enforce-module-boundaries
import { Item, ItemLayout, ItemProps } from '@clds/item';

import { NestedListItemsWrapper } from './common';

const ListItem: FC<ItemProps & { id: string }> = ({
  id,
  children,
  onClick,
  ...rest
}) => {
  const listStyle = useListStyle();
  usePubSubSubscribe(id, onClick as () => void);
  const { onClick: onClickListSelection, ...selection } = useListSelection(id);

  const onClickCombined = mergeCallbacks(
    onClickListSelection,
    onClick as () => void,
  );

  return (
    <Item {...rest} {...listStyle} {...selection} onClick={onClickCombined}>
      {children}
    </Item>
  );
};

const Demo = () => {
  const [selectedId, setSelectedId] = useState<string | Unselect>(unselect);
  const ids = [
    'settings',
    'transformations',
    'analyze',
    'edit',
    'extension',
    'share',
  ];

  const clickHandlers = usePubSub();

  const confirm = () => {
    if (selectedId !== unselect) {
      clickHandlers.publish(selectedId);
    }
  };
  const cancel = action('cancel');

  const handlers: KeyboardHandlers = {
    ...useListNavigationHandlers({
      items: ids,
      activeItem: selectedId,
      onActiveItemChange: setSelectedId,
    }),
    onEnter: confirm,
    onSpace: confirm,
    onEscape: cancel,
  };
  const { focusableProps } = useListNavigation(handlers);

  return (
    <PubSubProvider pubSub={clickHandlers}>
      <p>
        <code>selectedId: {String(selectedId)}</code>
      </p>
      <ListSelectionProvider onClick={setSelectedId} selectedId={selectedId}>
        <ListStyleProvider borderRadius="md">
          <DefaultList {...focusableProps}>
            <ListItem id={ids[0]} onClick={action('manage clicked')}>
              <ItemLayout leftSlot={<Settings />}>Manage</ItemLayout>
            </ListItem>
            <ListItem
              id={ids[1]}
              onClick={action('edit transformations clicked')}
            >
              <ItemLayout leftSlot={<Transform />}>
                Edit Transformations
              </ItemLayout>
            </ListItem>
            <ListItem id={ids[2]} onClick={action('analyze clicked')}>
              <ItemLayout leftSlot={<FaceDetect />}>Analyze</ItemLayout>
            </ListItem>
            <NestedListItemsWrapper>
              <p>
                {/* eslint-disable-next-line react/no-unescaped-entities */}
                You can see that list items don't need to be immediate children
                of List component
              </p>
              <ListItem id={ids[3]} onClick={action('edit clicked')}>
                <ItemLayout leftSlot={<Edit />}>Edit</ItemLayout>
              </ListItem>
              <ListItem id={ids[4]} onClick={action('extension clicked')}>
                <ItemLayout leftSlot={<Extension />}>Extension</ItemLayout>
              </ListItem>
            </NestedListItemsWrapper>
            <ListItem id={ids[5]} onClick={action('share clicked')}>
              <ItemLayout leftSlot={<SharedCollection />}>Share</ItemLayout>
            </ListItem>
          </DefaultList>
        </ListStyleProvider>
      </ListSelectionProvider>
    </PubSubProvider>
  );
};

export const PubSub = () => (
  <>
    <h1>Communication with children</h1>
    <p>
      Connected components combined with navigation hook demonstrated in
      previous stage look better. At this stage we also want to solve problem
      appeared in previous stage: calling deep children onClick event
      programmatically from parent.
    </p>
    <p>
      This time we add simple pub-sub mechanism to communicate with deep
      children about onClick event.
    </p>
    <p>
      <strong>usePubSub</strong> hook creates a pubSub communication bus and
      shares it via context so the list items can subscribe messages. In this
      example we assign each pub-sub topic to individual list item by item id.
      When enter key is pressed we publish a message to the topic for selected
      id. List items subscribe their own topic and when they receive the message
      they simply fire own handler.
    </p>
    <p>
      Please <strong>look at actions panel</strong> to check list items onClick
      handlers are fired when enter key is pressed
    </p>
    <Demo />
    <h2>Drawbacks</h2>
    <p>
      The main problem at this stage is that the{' '}
      <strong>useListNavigation()</strong> hook requires{' '}
      <strong>full array of item ids</strong> in order to properly determine new
      item id after pressing up/down/home/end keys. This is not very handy as
      you need to create that array at the top of your component definition.
    </p>
  </>
);

const meta: Meta<typeof Demo> = {
  title: 'Compound Components/04 Communication with children',
  component: Demo,
};

export default meta;
