import { action } from '@storybook/addon-actions';
import { Meta } from '@storybook/react';
import React, { FC, useState } from 'react';

import {
  KeyboardHandlers,
  useListNavigationHandlers,
  DefaultList,
  ListSelectionProvider,
  useListSelection,
  ListStyleProvider,
  useListStyle,
  useListNavigation,
  mergeCallbacks,
  PubSubProvider,
  usePubSub,
  usePubSubSubscribe,
  useQuerySelectorAll,
  unselect,
  Unselect,
} from '@clds/compound-components';
// eslint-disable-next-line @nx/enforce-module-boundaries
import {
  Edit,
  Extension,
  FaceDetect,
  Settings,
  SharedCollection,
  Transform,
} from '@clds/icon';
// eslint-disable-next-line @nx/enforce-module-boundaries
import { Item, ItemLayout, ItemProps } from '@clds/item';

import { NestedListItemsWrapper } from './common';

const ListItem: FC<ItemProps & { id: string }> = ({
  id,
  children,
  onClick,
  ...rest
}) => {
  const listStyle = useListStyle();
  usePubSubSubscribe(id, onClick as () => void);
  const { onClick: onClickListSelection, ...selection } = useListSelection(id);

  const onClickCombined = mergeCallbacks(
    onClickListSelection,
    onClick as () => void,
  );

  return (
    <Item
      {...rest}
      {...listStyle}
      data-selectable-id={id}
      {...selection}
      onClick={onClickCombined}
    >
      <ItemLayout>{children}</ItemLayout>
    </Item>
  );
};

const Demo = () => {
  const [includeAdditionalItem, setIncludeAdditionalItem] = useState(false);
  const onToggle = () => setIncludeAdditionalItem(!includeAdditionalItem);

  const [selectedId, setSelectedId] = useState<string | Unselect>(unselect);
  const { ref: querySelectorRef, result: listItemsElements } =
    useQuerySelectorAll('[data-selectable-id');
  const ids = listItemsElements.map(
    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
    (element) => element.getAttribute('data-selectable-id') || '',
  );
  const clickHandlers = usePubSub();

  const confirm = () => {
    if (selectedId !== unselect) {
      clickHandlers.publish(selectedId);
    }
  };
  const cancel = action('cancel');

  const handlers: KeyboardHandlers = {
    ...useListNavigationHandlers({
      items: ids,
      activeItem: selectedId,
      onActiveItemChange: setSelectedId,
    }),
    onEnter: confirm,
    onSpace: confirm,
    onEscape: cancel,
  };
  const { focusableProps } = useListNavigation(handlers);

  return (
    <PubSubProvider pubSub={clickHandlers}>
      <p data-chromatic="ignore">
        <code>selectedId: {String(selectedId)}</code>
      </p>
      <label>
        <input
          type="checkbox"
          onChange={onToggle}
          checked={includeAdditionalItem}
        />
        <span> Render additional items</span>
      </label>
      <ListSelectionProvider onClick={setSelectedId} selectedId={selectedId}>
        <ListStyleProvider borderRadius="md">
          <DefaultList {...focusableProps} ref={querySelectorRef}>
            <ListItem id="manage" onClick={action('manage clicked')}>
              <ItemLayout leftSlot={<Settings />}>Manage</ItemLayout>
            </ListItem>
            <ListItem
              id="edit-transformations"
              onClick={action('edit transformations clicked')}
            >
              <ItemLayout leftSlot={<Transform />}>
                Edit Transformations
              </ItemLayout>
            </ListItem>
            <ListItem id="analyze" onClick={action('analyze clicked')}>
              <ItemLayout leftSlot={<FaceDetect />}>Analyze</ItemLayout>
            </ListItem>
            {includeAdditionalItem && (
              <NestedListItemsWrapper>
                <p>
                  {/* eslint-disable-next-line react/no-unescaped-entities */}
                  You can see that list items don't need to be immediate
                  children of List component
                </p>
                <ListItem id="edit" onClick={action('edit clicked')}>
                  <ItemLayout leftSlot={<Edit />}>Edit</ItemLayout>
                </ListItem>
                <ListItem id="extension" onClick={action('extension clicked')}>
                  <ItemLayout leftSlot={<Extension />}>Extension</ItemLayout>
                </ListItem>
              </NestedListItemsWrapper>
            )}
            <ListItem id="share" onClick={action('share clicked')}>
              <ItemLayout leftSlot={<SharedCollection size="md" />}>
                Share
              </ItemLayout>
            </ListItem>
          </DefaultList>
        </ListStyleProvider>
      </ListSelectionProvider>
      Ids:
      <ul>
        {ids.map((k) => (
          <li key={k}>{k}</li>
        ))}
      </ul>
    </PubSubProvider>
  );
};

export const ImplicitIds = () => (
  <>
    <h1>
      Get rid of explicit <em>ids</em> array
    </h1>
    <p>
      In previous stage we managed to programatically fire events defined on
      deep children list items components. The most painful thing right now is
      that you have to maintain explicit list of menu item ids. We will make
      this implicit by calculating the array from deeply defined menu items id
      props.
    </p>
    <p>
      {/* eslint-disable-next-line react/no-unescaped-entities */}
      To achieve this we can't simply traverse children props as virtual dom
      {/* eslint-disable-next-line react/no-unescaped-entities */}
      doesn't represent final list structure. This may be different when you use
      routing or just wrap menu items inside another component.
    </p>
    <p>
      In this example we rely on rendered dom structure and get array ids using
      element.querySelectorAll() api. To achieve this we need our list items to
      render certain data-selectable-id attribute. Then we call{' '}
      <strong>useQuerySelectorAll()</strong> hook to get ids of rendered items
      in correct order.
    </p>
    <Demo />
    <h2>Drawbacks</h2>
    <p>
      {/* eslint-disable-next-line react/no-unescaped-entities */}
      If we don't need to maintain explicit list of ids upfront, setting id for
      each list item looks inconvenient.
    </p>
  </>
);

const meta: Meta<typeof Demo> = {
  title: 'Compound Components/05 Implicit ids array',
  component: Demo,
};

export default meta;
