import { action } from '@storybook/addon-actions';
import { Meta } from '@storybook/react';
import React, { FC, useState } from 'react';

import {
  KeyboardHandlers,
  useListNavigationHandlers,
  DefaultList,
  ListSelectionProvider,
  useListSelection,
  ListStyleProvider,
  useListStyle,
  useListNavigation,
  mergeCallbacks,
  PubSubProvider,
  usePubSub,
  usePubSubSubscribe,
  useQuerySelectorAll,
  useUniqueId,
  unselect,
  Unselect,
} from '@clds/compound-components';
// eslint-disable-next-line @nx/enforce-module-boundaries
import {
  Edit,
  Extension,
  FaceDetect,
  Settings,
  SharedCollection,
  Transform,
} from '@clds/icon';
// eslint-disable-next-line @nx/enforce-module-boundaries
import { Item, ItemLayout, ItemProps } from '@clds/item';

import { NestedListItemsWrapper } from './common';

const ListItem: FC<ItemProps> = ({ children, onClick, ...rest }) => {
  const listStyle = useListStyle();
  const id = useUniqueId();
  usePubSubSubscribe(id, onClick as () => void);
  const { onClick: onClickListSelection, ...selection } = useListSelection(id);

  const onClickCombined = mergeCallbacks(
    onClickListSelection,
    onClick as () => void /*onClick from list item can expect event but in our case we know it is not*/,
  );

  return (
    <Item
      {...rest}
      {...listStyle}
      data-selectable-id={id}
      {...selection}
      onClick={onClickCombined}
    >
      {children}
    </Item>
  );
};

const Demo = () => {
  const [includeAdditionalItem, setIncludeAdditionalItem] = useState(false);
  const onToggle = () => setIncludeAdditionalItem(!includeAdditionalItem);

  const [selectedId, setSelectedId] = useState<string | Unselect>(unselect);
  const { ref: querySelectorRef, result: listItemsElements } =
    useQuerySelectorAll('[data-selectable-id');
  const ids = listItemsElements.map(
    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
    (element) => element.getAttribute('data-selectable-id') || '',
  );

  const clickHandlers = usePubSub();

  const confirm = () => {
    if (selectedId !== unselect) {
      clickHandlers.publish(selectedId);
    }
  };
  const cancel = action('cancel');

  const handlers: KeyboardHandlers = {
    ...useListNavigationHandlers({
      items: ids,
      activeItem: selectedId,
      onActiveItemChange: setSelectedId,
    }),
    onEnter: confirm,
    onSpace: confirm,
    onEscape: cancel,
  };
  const { focusableProps } = useListNavigation(handlers);

  return (
    <PubSubProvider pubSub={clickHandlers}>
      <p data-chromatic="ignore">
        <code>selectedId: {String(selectedId)}</code>
      </p>
      <label>
        <input
          type="checkbox"
          onChange={onToggle}
          checked={includeAdditionalItem}
        />
        <span> Render additional items</span>
      </label>
      <ListSelectionProvider onClick={setSelectedId} selectedId={selectedId}>
        <ListStyleProvider borderRadius="md">
          <DefaultList {...focusableProps} ref={querySelectorRef}>
            <ListItem onClick={action('manage clicked')}>
              <ItemLayout leftSlot={<Settings />}>Manage</ItemLayout>
            </ListItem>
            <ListItem onClick={action('edit transformations clicked')}>
              <ItemLayout leftSlot={<Transform />}>
                Edit Transformations
              </ItemLayout>
            </ListItem>
            <ListItem onClick={action('analyze clicked')}>
              <ItemLayout leftSlot={<FaceDetect />}>Analyze</ItemLayout>
            </ListItem>
            {includeAdditionalItem && (
              <NestedListItemsWrapper>
                <p>
                  {/* eslint-disable-next-line react/no-unescaped-entities */}
                  You can see that list items don't need to be immediate
                  children of List component
                </p>
                <ListItem onClick={action('edit clicked')}>
                  <ItemLayout leftSlot={<Edit />}>Edit</ItemLayout>
                </ListItem>
                <ListItem onClick={action('extension clicked')}>
                  <ItemLayout leftSlot={<Extension />}>Extension</ItemLayout>
                </ListItem>
              </NestedListItemsWrapper>
            )}
            <ListItem onClick={action('share clicked')}>
              <ItemLayout leftSlot={<SharedCollection size="md" />}>
                Share
              </ItemLayout>
            </ListItem>
          </DefaultList>
        </ListStyleProvider>
      </ListSelectionProvider>
      Ids:
      <ul data-chromatic="ignore">
        {ids.map((k) => (
          <li key={k}>{k}</li>
        ))}
      </ul>
    </PubSubProvider>
  );
};

export const NoIds = () => (
  <>
    <h1>Get rid of ids at all</h1>
    <p>
      The last thing we can improve is to let list items define ids by their
      own. Automatically.
    </p>
    <p>
      To achieve this we can use <strong>useUniqueId()</strong> hook that keeps
      unique id which is persistent during whole list item lifecycle
    </p>
    <p>
      Please use <kbd>up</kbd>, <kbd>down</kbd>, <kbd>home</kbd>, <kbd>end</kbd>{' '}
      keys and observe how selection changes. Please
      <strong> look at actions panel</strong> to check that list items onClick
      handlers are fired when they are clicked or the enter key is pressed. This
      works without defining any single item id.
    </p>
    <Demo />
  </>
);

const meta: Meta<typeof Demo> = {
  title: 'Compound Components/06 No ids at all',
  component: Demo,
};

export default meta;
